<?php

// -------------------------------------------
// Espresso ELN Batch Registration Interface 
// -------------------------------------------

// Version 1.0.1 by ChemBytes

// This is a sample PHP script returning demo values to the ELN without interacting with an existing batch 
// registration system. If needs to be modified towards the specific API of your batch registration system 
// it interacts with. See the Espresso ELN 10 help file under the 'Product Registration Script' topic 
// for more details.


// -----------------------------------------------------
// Retrieve passed data and select the required action.
// (do NOT modify!)
// -----------------------------------------------------

$httpMethod = $_SERVER['REQUEST_METHOD'];
$payload = @file_get_contents('php://input'); 
$payloadInfo = json_decode($payload);

switch ($httpMethod) {

  case 'GET':
    GetRegistrationInfo();  // get registration system info
    break;

  case 'POST':
    RegisterBatch($payloadInfo,false);  // create new product batch
    break;

  case 'PUT':
    RegisterBatch($payloadInfo,true);  // update product batch
    break;

  case 'DELETE':
    RequestDeleteBatch($payloadInfo); // delete product batch
    break;
}


// -----------------------------------------------
// Provide product batch registration system info
// -----------------------------------------------

// Typically called by the ELN at startup only, so restart the ELN to make changes effective.

function GetRegistrationInfo(){

  // Modify according to your registration system (most will be structure-based)
  $isStructureBased = true;     // Set to true if the generated batch-ID's are structure-based, e.g. prefix-structNr-batchNr
  $useSDFileOnly = false;       // Set to true if only SD-file based information can be processed
  // ------------------

  $result = json_encode(array(
    'IsStructureBased' => $isStructureBased,
    'SDFileOnly' => $useSDFileOnly   
   ));

  http_response_code(200);  
  echo($result); 

}
// --------------------------------------------------


// -------------------------------------
//  Register or update a product batch.
// -------------------------------------

function RegisterBatch($prodInfo,$isUpdating){

   // Retrieve posted product info (JSON)   
   $infoType = $prodInfo -> ContentType;

   if ($infoType == 'Properties'){

      // Retrieve the product batch properties provided by the ELN
      // ---------------------------------------------------------
      // All properties not annotated to possibly be unspecified are guaranteed to contain values.
      // Your registry API most likely will only require a subset of the properties provided below. 
      // A minimum set might consist of the molfile and the experimentID.

      $updateBatchID = $prodInfo -> UpdateBatchID;  // empty string if creating a new batch (default); otherwise the ID of the batch to be updated 
                                                    // (for structure-based batch-ID's only updates with unmodified structure are possible)  
      $molfile = $prodInfo -> Molfile;              // product molfile
      $experimentID = $prodInfo -> ExperimentID;    // the unique ID of the experiment producing the batch 
      $molweight = $prodInfo -> MW;                 // product molecular weight
      $inchiKey=$prodInfo -> InChIKey;              // the InChIKey of the product
      $elemental = $prodInfo -> EF;                 // elemental formula of the product
      $author = $prodInfo -> Author;                // unique ID of the person producing the batch
      $purity = $prodInfo -> Purity;                // batch purity in percent
      $grams = $prodInfo -> Grams;                  // the batch amount, scaled to grams
      $amount = $prodInfo -> Amount;                // user specified batch amount
      $unit = $prodInfo ->  Unit;                   // user specified batch unit ('mg', 'g', or 'kg')
      $projName = $prodInfo -> ProjectName;         // project name
      $physForm =  $prodInfo -> PhysicalForm;       // physical form; will be 'liquid', 'solid', 'crystals', 'oil', 'gum', 'foam' or 'unspecified'
      $ee = $prodInfo -> EE;                        // enantimeric excess (can be unspecified)
      $de = $prodInfo -> DE;                        // diastereomeric excess (can be unspecified)
      $mp_upper = $prodInfo -> MP_Upper;            // upper limit of the melting point range (can be unspecified)
      $mp_lower = $prodInfo -> MP_Lower;            // lower limit of the melting point range (can be unspecified)
      $bp_upper =  $prodInfo -> BP_Upper;           // upper limit of the boiling point range (can be unspecified)
      $bp_lower =  $prodInfo -> BP_Lower;           // lower limit of the boiling point range (can be unspecified)
      $bp_pressure =  $prodInfo -> BP_Pressure;     // boiling point pressure in mbar (can be unspecified)
      $color =  $prodInfo -> Color;                 // color of the batch (can be unspecified)
      $resinLoad =  $prodInfo -> ResinLoad;         // resin load in percent, if attached to a resin (unspecified if not resin-bound)

  } 
  
  elseif ($infoType == 'SDFile'){

    // All product batch properties are delivered as SDFile content 
    // ------------------------------------------------------------

    // The SDFile contains the same batch properties as listed above.
    $SDFileStr = $prodInfo -> SDFile;

  };
  

  // Your custom code section for create/update starts here:
  //------------------------------------------------------------------

  if($isUpdating == false){

    // Your API call(s) for CREATING a new product batch based on above product data:
    // ......   
    
    $batchID = 'mySampleID';         // replace by actually returned batchID string          
    http_response_code(201);        // 201: The batch was successfully created (action always must be immediate)
                                    // 400: An error was returned by the registration API, the action failed.  

  } else {

    // Your API call(s) for UPDATING the product batch specified by $updateBatchID based on above product data 
    // .........

    $batchID = $updateBatchID;  
    http_response_code(201);        // 201: The batch was immediately updated.
                                    // 202: An update request was placed to a staging area for later processing. 
                                    // 400: An error was returned by the registration API, the action failed.  

  }

  // ----------------------------------------------

  echo($batchID); 
  
}


// ----------------------------------------------
// Delete a registered product batch 
// ----------------------------------------------

function  RequestDeleteBatch($cancelInfo){

    $batchID = $cancelInfo -> BatchID;			      // ID of the batch to deleted
    $cancelReason = $cancelInfo -> CancelReason;  // reason for deletion ('user_revoked', 'product_deleted', 'structure_deleted' or 'structure_modified') 
    $author = $cancelInfo -> Author;              // requesting user (full name) 
    $userID = $cancelInfo -> UserID;              // userID of the requesting user
    $experimentID = $cancelInfo -> ExperimentID;  // experimentID the product batch belongs to
    
   
    // Your custom code section for product batch delete starts here:
    //--------------------------------------------------------------------

    // Add here your API call(s) for deleting a batch.
    // ....................
   
    http_response_code(204);     // 204: The batch was immediately deleted.
                                 // 202: A delete request was placed to a staging area and will be processed later.
                                 // 400: An error was returned by the registration API, the action failed.  

    // ----------------------------------

    return;

}


?>
